;       SOME MACROS TO MAKE THE JOB EASIER

PRINT           macro           ;USAGE: PRINT MSG1
                push ax         ;ALL REGISTERS PRESERVED
                push dx         ;PRINTS STRING MSG1 AT CURRENT
                mov ah,9        ;CURSOR POSITION
                lea dx, #1
                int 021
                pop dx
                pop ax
                #em

PRINT$          macro           ;USAGE: PRINT$,"MESSAGE"
                push ax         ;PRINTS AT CURRENT CURSOR POSITION
                push dx  
                JMP >M1
M0              DB #1
                db "$"
M1:             mov ah,9        ;ALL REGISTERS PRESERVED
                lea dx,m0
                int 021
                pop dx
                pop ax
                #em

QUIT            MACRO             ;USAGE: QUIT 0
                MOV AH,04C        ;MUST GIVE SOME VALUE
                MOV AL,#1
                INT 021
                #EM

NEWLINE         macro           ;equivalent to cr,lf
                push ax         ;scrolls if off screen
                push dx
                jmp > m1
m0              db      0d,0a,'$'
m1:             lea dx,m0
                mov ah,9
                int 021
                pop dx
                pop ax
                #em


TEST_INT        EQU     060             ;DUMMY (UNUSED) INTERRUPT
                                        ;WE'LL USE THIS AS A SIGNATURE
                                        ;CHANGE TO SOMETHING ELSE IF
                                        ;INT 60H IN USE AND REASSEMBLE
                                        ;OR ALTER BYTES W/DEBUG OR SOMETHING

CR              EQU     0D              ;CARRIAGE RETURN
LF              EQU     0A              ;LINE FEED
SPACE           EQU     020

                ORG 0100
KEYCOUNT:       JMP LONG > L0
SEMAPHORE       DB      0
COUNT1          DW      0
COUNT2          DW      0
OLD_INT_9_1     DW      0
OLD_INT_9_2     DW      0
NEW_TEST_INT    DW      09090
NEW_INT_9:      STI
        CS      CMP [SEMAPHORE],1
                JE > L5
                PUSH AX
                PUSH BX
        CS      MOV AX,[COUNT1]
        CS      MOV BX,[COUNT2]
                ADD AX,1
                ADC BX,0
        CS      MOV COUNT1,AX
        CS      MOV COUNT2,BX
                POP BX
                POP AX
        CS      MOV [SEMAPHORE],1
                CLI
        CS      JMP D [OLD_INT_9_1]
L5:     CS      MOV [SEMAPHORE],0
        CS      JMP D [OLD_INT_9_1]     ;END OF TSR CODE


MSG1            DB      CR,LF,'The total keystroke count is $'
COUNT_AREA      DB      12 DUP(020)
END_OF_COUNT    DB      ' strokes since bootup',CR,LF,'$'

;       MAIN ENTRY POINT FOR INSTALLATION AND REPORTS

L0:             MOV AH,035              ;FIND OUT IF WE'RE ACTIVE
                MOV AL,TEST_INT         ;PATCH POINT FOR TEST_INT CHANGE       
                INT 021
        ES      MOV AX,[BX]
                CMP AX,09090            ;IS IT THE PROPER SIGNATURE?
                JNE > L1                ;IF NOT, INSTALL THE PROGRAM
        ES      MOV AX,[COUNT1]         ;IF WE'RE ALREADY INSTALLED,
        ES      MOV DX,[COUNT2]         ;CALCULATE THE COUNT 
                LEA DI,END_OF_COUNT     ;NOTE THAT WE CAN USE ES IN PLACE OF DS
                DEC DI                  ;BECAUSE OF THE WAY THE PROGRAM IS
                PUSH DI                 ;SET UP.  CLEVER, NO?
                CALL LARGE_HEX_TO_ASCII ;CONVERT THE 32-BIT NUMBER TO ASCII
                PRINT MSG1              ;AND PRINT THE MESSAGE BEGINNING
                MOV SI,DI
                POP DI
                CALL PUT_COMMAS         ;PUT THE COMMAS IN THE NUMBER STRING
                MOV DX,SI
                MOV AH,9
                INT 021                 ;AND PRINT THE REST OF THE MESSAGE
                QUIT 0                  ;EXIT WITH ERROR CODE 0


L1:             MOV AH,025              ;IN HERE IF WE'RE INSTALLING
                MOV AL,TEST_INT         ;GRAB DUMMY INTERRUPT
                LEA DX,NEW_TEST_INT
                INT 021                 ;AND PUT IT IN THE CHAIN
                MOV AX,03509            ;GET ADDRESS OF INT 09H
                INT 021
                MOV [OLD_INT_9_1],BX    ;SAVE ITS ADDRESS
                MOV [OLD_INT_9_2],ES
                MOV AX,02509            ;GOING TO PUT NEW INT 09H
                LEA DX,NEW_INT_9        ;POINTING TO ME
                INT 021
                PRINT$ 'Installing key count utility'  ;TELL'EM WE'RE IN
                NEWLINE
                LEA DX,MSG1             ;GET ADDRESS OF END OF TSR
                INT 027                 ;TERMINATE-AND-STAY-RESIDENT
                END

;               SUPPORTING SUBROUTINES


;       ROUTINE TO CONVERT A 32-BIT NUMBER TO A DECIMAL ASCII STRING
;       ENTER WITH THE NUMBER TO BE CONVERTED IN DX:AX
;       AND DI POINTING TO THE LAST BYTE OF THE RESULT STRING AREA
;       RETURN WITH DI POINTING TO FIRST NON-ZERO CHARACTER OF RESULTANT
;       CONVERSION.
;
;       BX, CX, and SI unchanged
;
;       This routine was cribbed from FREE by Art Merrill - I don't really
;       understand it all, but it works!

LARGE_HEX_TO_ASCII:     PUSH BX
                        PUSH CX
                        XCHG CX,DX
                        MOV BX,10
L1:                     CMP CX,0
                        JE > L2
                        XCHG AX,CX
                        XOR DX,DX
                        DIV BX
                        XCHG AX,CX
                        DIV BX
                        OR DL,030
                        MOV [DI],DL
                        DEC DI
                        JMP L1
L2:                     XOR DX,DX
                        DIV BX
                        OR DL,030
                        MOV [DI],DL
                        DEC DI
                        CMP AX,0
                        JNE L2
                        INC DI          ;BACK UP POINTER TO FIRST CHAR
                        POP CX
                        POP BX
                        RET

;       ROUTINE TO PUT COMMAS IN A NUMERIC STRING
;
;       ENTER WITH SI POINTING TO FIRST CHARACTER OF STRING
;                  DI POINTS TO LAST CHARACTER OF STRING
;
;       RETURNS WITH SI, DI POINTING TO FIRST AND LAST CHARACTERS OF
;       (NEW) STRING.
;
;       ALL OTHER REGISTERS UNCHANGED.
;
;       NOTE: THERE MUST BE SUFFICIENT ROOM TO LEFT OF STRING TO HOLD 
;       NEW (SHIFTED) STRING, AND THE SPARE ROOM MUST BE FILLED
;       WITH SPACES.
;
;       NO CHECK IS MADE TO SEE IF STRING IS REALLY NUMERIC, NOR ARE
;       ANY ERROR CONDITIONS DETECTED.

PUT_COMMAS:     PUSH AX
                PUSH DI
                MOV AX,DI
                SUB AX,SI
                CMP AX,2
                JBE > L1
L0:             SUB DI,3
                MOV AH,[DI]
                CMP AH,SPACE
                JE > L1
                CALL SHFT_STR_LFT
                MOV B[DI],','
                DEC DI
                JMP L0
L1:             POP DI
                POP AX
                RET

;
;       ROUTINE TO SHIFT A STRING LEFT ONE POSITION
;
;       ENTER WITH DI POINTING TO THE START OF THE SHIFT
;       SI POINTS TO LEFTMOST CHARACTER OF STRING.
;
;       ON RETURN, DI POINTS TO ORIGINAL POSITION (NOW DUPLICATED)
;       AND SI POINTS TO LEFTMOST CHARACTER OF STRING.
;

SHFT_STR_LFT:   PUSH CX
                PUSH AX
                PUSH SI
                MOV CX,DI
                SUB CX,SI
                INC CX
L1:             MOV AH,[SI]
                MOV [SI-1],AH
                INC SI
                LOOP L1
                POP SI
                DEC SI
                POP AX
                POP CX
                RET

